from parallels.core import messages
from parallels.core.application_adjuster.application_configs.base import ApplicationConfig
from parallels.core.application_adjuster.application_configs.utils import path_endswith, replace_line
from parallels.core.utils.common import safe_format


class WordpressApplicationConfig(ApplicationConfig):
    """Class for detection and fixing Wordpress configuration file"""

    def get_config_file_description(self, filename):
        """Human-readable description of configuration file

        :rtype: str | unicode
        """
        return safe_format(
            messages.APPLICATION_FILE_CONFIG_DESCRIPTION,
            filename=filename, application_name='WordPress'
        )

    def filename_match(self, path):
        """Check if given path looks like configuration file of Wordpress

        :type path: str | unicode
        :rtype: boolean
        """
        return path_endswith(path, 'wp-config.php')

    def contents_match(self, contents):
        """Check if given file contents looks like configuration file of Wordpress

        :type contents: str | unicode
        :rtype: boolean
        """
        return True

    def fix_database_hostname(self, contents, db_host_mapping):
        """Fix database hostname in configuration file of Wordpress

        :type contents: str | unicode
        :type db_host_mapping: dict[str | unicode, str | unicode]
        :rtype: parallels.core.utils.line_processor.ReplaceResults
        """
        # default config has line like:
        # define('DB_HOST', '10.52.143.116');
        return replace_line(contents, 'DB_HOST', "'DB_HOST', '", "'", db_host_mapping)